@@ -1,11 +1,10 @@
+use crate::{api::unsafe_buffer, error_hook};
 use elrond_wasm::{
     api::{Handle, InvalidSliceError, ManagedBufferApi},
     err_msg,
     types::BoxedBytes,
 };
 
-use crate::error_hook;
-
 // #[allow(dead_code)]
 extern "C" {
     fn mBufferNew() -> i32;
@@ -165,3 +164,9 @@ impl ManagedBufferApi for crate::ArwenApiImpl {
         unsafe { mBufferEq(handle1, handle2) > 0 }
     }
 }
+
+pub(crate) unsafe fn unsafe_buffer_load_address(address_handle: Handle) -> *const u8 {
+    let unsafe_buffer_ptr = unsafe_buffer::buffer_ptr();
+    let _ = mBufferGetBytes(address_handle, unsafe_buffer_ptr);
+    unsafe_buffer_ptr
+}
